#   Copyright 2011-2016 Josh Kearney
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Pyhole Logging"""

import bz2
import glob
import logging
import logging.handlers
import os
import requests
import shutil

import utils


LOG_DIR = utils.get_directory("logs")
LOG_ARCHIVE_DIR = utils.get_directory(os.path.join("logs", "archive"))
LOG_FORMAT = "%(asctime)s [%(name)s] %(message)s"
LOG_DATEFMT = "%H:%M:%S"


class PyholeFileHandler(logging.handlers.TimedRotatingFileHandler):
    def doRollover(self):
        result = super(PyholeFileHandler, self).doRollover()
        self.archive_old_logs()
        return result

    def archive_old_logs(self):
        matcher = "*.log.*[!b][!z][!2]"
        files = glob.glob(os.path.join(LOG_DIR, matcher))
        for file_path in files:
            filename = os.path.basename(file_path)
            compressed_filename = filename + ".bz2"
            network_name = filename[:filename.rfind(".log")]
            archive_dir = os.path.join(LOG_ARCHIVE_DIR, network_name)
            utils.make_directory(archive_dir)
            compressed_file_path = os.path.join(archive_dir,
                                                compressed_filename)

            with open(file_path, "rb") as fp:
                with bz2.BZ2File(compressed_file_path, "wb",
                                 compresslevel=9) as output:
                    shutil.copyfileobj(fp, output)

            os.remove(file_path)


def setup_logger(name):
    """Setup the logger."""
    # NOTE(jk0): Disable unnecessary requests logging.
    requests.packages.urllib3.disable_warnings()
    requests_log = logging.getLogger("requests")
    requests_log.setLevel(logging.WARNING)

    debug = utils.debug_enabled()

    log_level = logging.DEBUG if debug else logging.INFO

    logging.basicConfig(level=log_level,
                        format=LOG_FORMAT,
                        datefmt=LOG_DATEFMT)

    log_file = os.path.join(LOG_DIR, name.lower() + ".log")
    log = PyholeFileHandler(log_file, "midnight")
    log.setLevel(log_level)
    formatter = logging.Formatter(LOG_FORMAT, LOG_DATEFMT)
    log.setFormatter(formatter)
    logging.getLogger(name).addHandler(log)

    log.archive_old_logs()


def get_logger(name="Pyhole"):
    return logging.getLogger(name)



#   Copyright 2010-2016 Josh Kearney
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Pyhole Utilies"""

import argparse
import datetime
import os
import re
import shutil
import threading

from BeautifulSoup import BeautifulStoneSoup

import config
import version


def admin(func):
    """Require admin rights."""
    def wrap(self, message, *args, **kwargs):
        if message.source in self.session.admins:
            return func(self, message, *args, **kwargs)
        return message.dispatch("Sorry, you are not authorized to do that.")

    wrap.__doc__ = func.__doc__
    wrap.__name__ = func.__name__
    wrap.__module__ = func.__module__

    return wrap


def require_params(func):
    """Require parameters."""
    def wrap(self, message, params, *args, **kwargs):
        if not params:
            message.dispatch(wrap.__doc__)
            return
        return func(self, message, params, *args, **kwargs)

    wrap.__doc__ = func.__doc__
    wrap.__name__ = func.__name__
    wrap.__module__ = func.__module__

    return wrap


def spawn(func):
    """Thread-spawning decorator."""
    def wrap(*args, **kwargs):
        t = threading.Thread(target=func, args=args, kwargs=kwargs)
        t.setDaemon(True)
        t.start()

    wrap.__doc__ = func.__doc__
    wrap.__name__ = func.__name__
    wrap.__module__ = func.__module__

    return wrap


def decode_entities(html):
    """Strip HTML entities from a string and make it printable."""
    html = " ".join(str(x).strip() for x in BeautifulStoneSoup(html,
                    convertEntities=BeautifulStoneSoup.HTML_ENTITIES).findAll(
                    text=True))
    return filter(lambda x: ord(x) > 9 and ord(x) < 127, html)


def ensure_int(param):
    """Ensure the given param is an int."""
    try:
        param = re.sub("\W", "", param)
        return int(param)
    except ValueError:
        return None


def build_options():
    """Generate command line options."""
    parser = argparse.ArgumentParser(version=version.version_string())
    parser.add_argument("-c", "--config", default=get_conf_file_path(),
                        help="specify the path to a configuration file")
    parser.add_argument("-d", "--debug", action="store_true",
                        help="show debugging output")
    return parser.parse_known_args()[0]


def get_option(option):
    """Retrive an option from the command line."""
    parsed_args = build_options()
    return vars(parsed_args).get(option)


def debug_enabled():
    """Return whether or not debug mode is enabled."""
    debug_option = get_option("debug")
    debug_config = get_config().get("debug", type="bool")
    return debug_option or debug_config


def get_home_directory():
    """Return the home directory."""
    home_dir = os.getenv("HOME") + "/.pyhole/"
    make_directory(home_dir)
    return home_dir


def get_directory(new_dir):
    """Return a directory."""
    home_dir = get_home_directory()
    new_dir = os.path.join(home_dir, new_dir, "")
    make_directory(new_dir)
    return new_dir


def make_directory(directory):
    """Make a direectory if it doesn't exist."""
    if not os.path.exists(directory):
        os.makedirs(directory)


def get_conf_file_path():
    """Return the path to the conf file."""
    paths = (
        "/etc/pyhole.conf",
        "/etc/pyhole/pyhole.conf"
    )

    for path in paths:
        if os.path.exists(path):
            return path

    return get_home_directory() + "pyhole.conf"


def get_conf_file():
    """Return the path to the conf file."""
    return get_option("config") or get_conf_file_path()


def get_config(section="Pyhole"):
    """Return the default config object."""
    return config.Config(get_conf_file(), section)


def write_file(directory, file_name, data):
    """Write data to file."""
    directory = get_directory(directory)
    with open(directory + file_name, "w") as open_file:
        open_file.write(str(data).strip())


def read_file(directory, file_name):
    """Read and return the data in file."""
    directory = get_directory(directory)
    try:
        with open(directory + file_name, "r") as open_file:
            data = open_file.read()
        return data
    except IOError:
        return None


def list_files(directory):
    """List files in a directory."""
    directory = get_directory(directory)
    return os.listdir(directory)


def prepare_config():
    """Prepare a sample configuration file."""
    conf_file = get_conf_file()
    if os.path.exists(conf_file):
        return

    try:
        print "Copying sample configuration file to: %s" % conf_file
        shutil.copyfile("pyhole.conf.sample", conf_file)
        print "Done. Please edit before running again."
    except IOError:
        # NOTE(jk0): Could not locate sample configuration file. This should
        # only happen when Read the Docs generates the documentation.
        pass


def datetime_now_string():
    """ISO 8601 formatted string of the current datetime."""
    return datetime.datetime.utcnow().isoformat()



#   Copyright 2016 Josh Kearney
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Pyhole RESTful API"""

import cgi
import flask
import os
import time
import uuid

from pyhole.core import queue
from pyhole.core import utils
from pyhole.core import version


APP = flask.Flask("pyhole")
QUEUE = queue.MessageQueue()


@APP.route("/", methods=["GET"])
def index():
    """Handle index requests."""
    return version.version_string(), 200


# BEGIN MESSAGE API #
@APP.route("/messages/send", methods=["POST"])
def send_message():
    """Send a message."""
    request = flask.request.get_json()

    try:
        item = (
            request["network"],
            request["target"],
            request["message"]
        )
    except KeyError:
        flask.abort(422)

    # NOTE(jk0): Disable until auth is implemented.
    # QUEUE.put(item)

    return str(item), 200
# END MESSAGE API #


# BEGIN PASTE API #
PASTE_TEMPLATE = """{% set lines = paste.split('\n') %}<html>
<head>
<title>{{ paste_id }} | pyhole</title>
<style type="text/css">
* {
    margin: 0;
    padding: 0;
}
body {
    margin: 20px 0;
    background-color: #fff;
    font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
}
pre {
    padding: 10px;
    font-family: Consolas, "Liberation Mono", Menlo, Courier, monospace;
    font-size: 12px;
    color: #333;
    letter-spacing: 0.5px;
    line-height: 160%;
    white-space: pre-wrap;
}
#paste {
    margin: 0 auto;
    width: 65%;
    border-radius: 3px;
    border: 1px solid #ddd;
}
#lines {
    float: left;
    width: 3%;
    color: #b3b3b3;
    text-align: right;
    border-right: 1px solid #eee;
}
#snippet {
    overflow: auto;
}
#header {
    border-bottom: 1px solid #ddd;
    padding: 15px;
    background-color: #f7f7f7;
    font-family: Consolas, "Liberation Mono", Menlo, Courier, monospace;
    font-weight: bold;
    font-size: 14px;
    color: #4078c0;
}
#header p {
    float: left;
    width: 94%;
}
#header a {
    padding: 5px 10px;
    border: 1px solid #d5d5d5;
    border-radius: 3px;
    background-color: #f7f7f7;
    font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
    font-size: 13px;
    text-decoration: none;
    color: #333;
}
#header a:hover {
    background-color: #e6e6e6;
    border: 1px solid #ccc;
}
#copyright {
    padding: 20px 0 0 0;
    font-size: 12px;
    color: #767676;
    text-align: center;
}
</style>
</head>
<body>
<div id="paste">
<div id="header">
<p>{{ st_mtime }} / {{ st_size }} bytes</p>
<a href="/pastes/{{ paste_id }}/raw">Raw</a>
</div>
<pre id="lines">{% for line in lines %}
{{ loop.index }}{% endfor %}</pre>
<pre id="snippet">{{ paste }}</pre>
</div>
<p id="copyright">{{ version }}</p>
</body>
</html>
"""


@APP.route("/pastes/<paste_id>", methods=["GET"])
@APP.route("/pastes/<paste_id>/<raw>", methods=["GET"])
def get_paste(paste_id, raw=None):
    """Fetch and return a paste."""
    stats = os.stat(utils.get_directory("pastes") + paste_id)
    st_mtime = time.ctime(stats.st_mtime)
    st_size = stats.st_size

    paste = utils.read_file("pastes", paste_id)

    if not paste:
        flask.abort(404)

    if raw:
        return flask.Response(paste, status=200, mimetype="text/plain")

    return flask.render_template_string(
        PASTE_TEMPLATE,
        paste_id=paste_id,
        paste=cgi.escape(paste),
        st_mtime=st_mtime,
        st_size=st_size,
        version=version.version_string())


@APP.route("/pastes", methods=["POST"])
def create_paste():
    """Create a new paste."""
    try:
        paste = flask.request.get_json()["paste"]
    except KeyError:
        flask.abort(422)

    file_name = str(uuid.uuid4()).replace("-", "")
    utils.write_file("pastes", file_name, paste)

    response = "%s/%s" % (flask.request.url, file_name)

    try:
        # NOTE(jk0): If these items exist, write them to the queue.
        item = (
            flask.request.get_json()["network"],
            flask.request.get_json()["target"],
            response
        )

        QUEUE.put(item)
    except KeyError:
        pass

    return flask.redirect(response)
# END PASTE API #


@utils.spawn
def run():
    """Run the flask process."""
    config = utils.get_config()

    kwargs = {
        "host": "0.0.0.0",
        "threaded": True
    }

    ssl_crt = config.get("api_ssl_crt", default="")
    ssl_key = config.get("api_ssl_key", default="")

    if os.path.exists(ssl_crt) and os.path.exists(ssl_key):
        kwargs["ssl_context"] = (ssl_crt, ssl_key)

    APP.run(**kwargs)



#   Copyright 2010-2016 Josh Kearney
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Pyhole Network Connections"""

import multiprocessing

from pyhole.core import utils


class Process(multiprocessing.Process):
    """A network connection."""
    def __init__(self, network):
        super(Process, self).__init__()
        self.network = network

    def run(self):
        """Run the network connection."""
        network_config = utils.get_config(self.network)
        if network_config.get("api_token", default=None):
            from pyhole.core.slack import client
        else:
            from pyhole.core.irc import client

        connection = client.Client(self.network)
        connection.start()



#   Copyright 2016 Josh Kearney
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Pyhole Message Queue"""

import multiprocessing

import utils


QUEUE = multiprocessing.Queue()


class MessageQueue(object):
    """Global message queue."""

    def __init__(self):
        self.queue = QUEUE

    def put(self, item):
        """Place an item into the queue."""
        self.queue.put_nowait(item)

    @utils.spawn
    def watch(self, session):
        """Watch the queue for incoming messages."""
        while True:
            item = self.queue.get()

            actual_network = session.log.name
            intended_network = item[0]

            # NOTE(jk0): If the intended network does not match the actual
            # network, place the item back into the queue until it gets
            # picked up by the intended network.
            if actual_network != intended_network:
                self.queue.put(item)
                continue

            session.reply(item[1], item[2])



#   Copyright 2016 Josh Kearney
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Pyhole Operations Plugin"""

import time

from pyhole.core import plugin
from pyhole.core import request
from pyhole.core import utils


class Ops(plugin.Plugin):
    """Manage operational responsibilities."""

    def __init__(self, session):
        self.session = session
        self.name = self.__class__.__name__

        pagerduty = utils.get_config("PagerDuty")
        self.subdomain = pagerduty.get("subdomain")
        self.api_key = pagerduty.get("api_key")
        self.integration_key = pagerduty.get("integration_key")

        self.api_headers = {
            "Authorization": "Token token=%s" % self.api_key,
            "Content-Type": "application/json"
        }
        self.user_cache = None
        self.reset_current_incident()

    @plugin.hook_add_command("oncall")
    @utils.spawn
    def oncall(self, message, params=None, **kwargs):
        """Show who is on call (ex: .oncall [<group>])."""
        url = "%s/api/v1/escalation_policies/on_call" % self.subdomain
        req = request.get(url, headers=self.api_headers,
                          params={"query": params})

        if request.ok(req):
            response_json = req.json()
            for policy in response_json["escalation_policies"]:
                message.dispatch(policy["name"])

                for level in policy["on_call"]:
                    message.dispatch("- Level %s: %s <%s>" % (
                                     level["level"],
                                     level["user"]["name"],
                                     level["user"]["email"]))
                # NOTE(jk0): This is needed to prevent rate-limiting.
                time.sleep(2)
        else:
            message.dispatch("Unable to fetch list: %d" % req.status_code)

    @plugin.hook_add_command("incident")
    @utils.require_params
    @utils.spawn
    def incident(self, message, params=None, **kwargs):
        """Manage incidents (ex: .incident <create|resolve|setkey>)."""
        param_split = params.split(" ")
        command = param_split.pop(0)
        if command == "create":
            description = " ".join(param_split)
            self.create_incident(message, description)
        elif command == "resolve":
            self.resolve_incident(message)
        elif command == "setkey":
            key = " ".join(param_split)
            self.set_key(message, key)

    @plugin.hook_add_command("note")
    @utils.require_params
    @utils.spawn
    def note(self, message, params=None, **kwargs):
        """Create a note for the current incident (ex: .note <message>)."""
        number = self.get_incident_number()
        if number is None:
            msg = "Could not get incident number for current incident."
            message.dispatch(msg)
            return

        url = "%s/api/v1/incidents/%s/notes" % (self.subdomain, number)
        user = self.find_user_like(message.source)
        if user is None:
            message.dispatch("Could not find PagerDuty user matching: %s" %
                             message.source)
            return

        data = {
            "note": {
                "content": params
            },
            "requester_id": user["id"]
        }

        req = request.post(url, headers=self.api_headers, json=data)

        if request.ok(req):
            message.dispatch("Noted.")
        else:
            message.dispatch("Unable to create note: %d" % req.status_code)
            message.dispatch(req.text)

    @plugin.hook_add_command("notes")
    @utils.spawn
    def notes(self, message, params=None, **kwargs):
        """List all notes in current incident."""
        number = self.get_incident_number()
        if number is None:
            msg = "Could not get incident number for current incident."
            message.dispatch(msg)
            return

        url = "%s/api/v1/incidents/%s/notes" % (self.subdomain, number)
        req = request.get(url, headers=self.api_headers)
        if request.ok(req):
            response_json = req.json()
            notes = response_json["notes"]
            message.dispatch("There are %d notes." % len(notes))

            for note in notes:
                message.dispatch("%s %s - %s" % (note["created_at"],
                                                 note["user"]["name"],
                                                 note["content"]))
        else:
            message.dispatch("Unable to fetch notes: %d" % req.status_code)

    @plugin.hook_add_command("lookup")
    @utils.require_params
    @utils.spawn
    def lookup(self, message, params=None, **kwargs):
        """Lookup user's phone numbers (ex: .lookup <name>."""
        url = "%s/api/v1/users?limit=1000" % self.subdomain
        response = request.get(url, headers=self.api_headers).json()
        results = []
        for user in response["users"]:
            if params in user["name"] or params in user["email"]:
                url = "%s/api/v1/users/%s/contact_methods" % (self.subdomain,
                                                              user["id"])
                response = request.get(url, headers=self.api_headers).json()
                for contact in response["contact_methods"]:
                    if contact["type"] == "phone":
                        results.append("%s (%s)" % (user["name"],
                                                    contact["phone_number"]))
        if len(results) > 0:
            message.dispatch(", ".join(results))
        else:
            message.dispatch("No results found: %s" % params)

    def create_incident(self, message, description):
        """Create a PagerDuty incident with specified description."""
        # would have been called url, but flake8 complains that line is 81 long
        u = "https://events.pagerduty.com/generic/2010-04-15/create_event.json"
        data = {
            "service_key": self.integration_key,
            "event_type": "trigger",
            "description": description,
            "incident_key": utils.datetime_now_string(),
            "details": {"triggered by": message.source}
        }

        req = request.post(u, json=data)
        response_json = req.json()
        if request.ok(req):
            self.set_incident_key(response_json["incident_key"])
            message.dispatch("Created incident with incident_key: %s" %
                             self.get_incident_key())
        else:
            message.dispatch("Error creating incident: %d" % req.status_code)
            message.dispatch(response_json["message"])

    def resolve_incident(self, message):
        """Close the current PagerDuty incident."""
        incident_id = self.get_incident_id()
        if incident_id is None:
            message.dispatch("Could not find ID for current incident.")
            return

        url = "%s/api/v1/incidents/%s/resolve" % (self.subdomain, incident_id)
        user = self.find_user_like(message.source)
        if user is None:
            message.dispatch("Could not find PagerDuty user: %s" %
                             message.source)
            return

        data = {"requester_id": user["id"]}

        req = request.put(url, headers=self.api_headers, json=data)

        if request.ok(req):
            self.reset_current_incident()
            message.dispatch("Resolved incident successfully.")
        else:
            message.dispatch("Could not resolve incident: %d" %
                             req.status_code)
            message.dispatch(req.text)

    def set_key(self, message, key):
        """Set the current incident key."""
        self.set_incident_key(key)
        message.dispatch("Key set to: %s" % key)

    def get_users(self):
        """Get PagerDuty users and fill the cache."""
        url = "%s/api/v1/users" % self.subdomain
        req = request.get(url, headers=self.api_headers)
        if request.ok(req):
            response_json = req.json()
            return response_json["users"]
        else:
            return None

    def find_user_like(self, query):
        """Finds a PagerDuty user matching the query (name/email)."""
        # if cache is empty, fill it
        if self.user_cache is None:
            self.user_cache = self.get_users()

        # if it's still empty, something's wrong
        if self.user_cache is not None:
            # search the names first
            for user in self.user_cache:
                if query in user["name"]:
                    return user
            # then search the emails
            for user in self.user_cache:
                if query in user["email"]:
                    return user
        return None

    def reset_current_incident(self):
        """Resets the current incident state."""
        self.current_incident = {
            "key": None,
            "number": None,
            "id": None
        }

    def get_incident_key(self):
        return self.current_incident["key"]

    def get_incident_number(self):
        number = self.current_incident["number"]
        if number is None:
            number = self.find_incident_number()
            self.set_incident_number(number)
        return number

    def get_incident_id(self):
        the_id = self.current_incident["id"]
        if the_id is None:
            new_id = self.find_incident_id()
            self.set_incident_id(new_id)
        return the_id

    def set_incident_key(self, key):
        self.current_incident["key"] = key

    def set_incident_number(self, number):
        self.current_incident["number"] = number

    def set_incident_id(self, new_id):
        self.current_incident["id"] = new_id

    def find_incident_number(self):
        """Finds incident number from the current incident key."""
        key = self.get_incident_key()
        if key is not None:
            url = "%s/api/v1/incidents" % self.subdomain
            params = {"incident_key": key, "sort_by": "created_on:desc"}
            req = request.get(url, headers=self.api_headers, params=params)

            if request.ok(req):
                response_json = req.json()
                incidents = response_json["incidents"]
                if len(incidents) > 0:
                    number = incidents[0]["incident_number"]
                    return number
        return None

    def find_incident_id(self):
        """Finds incident id from the current incident number."""
        number = self.get_incident_number()
        if number is not None:
            url = "%s/api/v1/incidents/%s" % (self.subdomain, number)
            req = request.get(url, headers=self.api_headers)
            if request.ok(req):
                response_json = req.json()
                return response_json["id"]
        return None



#   Copyright 2010-2016 Josh Kearney
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""pyhole - A modular IRC & Slack bot."""

import time

from pyhole.core import api
from pyhole.core import logger
from pyhole.core import process
from pyhole.core import utils
from pyhole.core import version


def Main():
    """Main loop."""
    config = utils.get_config()
    log = logger.get_logger()
    networks = config.get("networks", type="list")

    log.info("Starting %s..." % version.version_string())
    log.info("Connecting to networks: %s" % ", ".join(networks))

    procs = []
    for network in networks:
        proc = process.Process(network)
        proc.start()
        procs.append(proc)

    try:
        if config.get("api_enabled", type="bool"):
            api.run()

        while True:
            time.sleep(1)
            for proc in procs:
                if not proc.is_alive():
                    procs.remove(proc)

            if not procs:
                log.info("No longer connected to any networks; shutting down.")
                raise KeyboardInterrupt
    except KeyboardInterrupt:
        for proc in procs:
            proc.terminate()



#   Copyright 2016 Patrick Tsai
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Pyhole Request Manager"""

import requests

import version


session = requests.Session()
session.headers.update({
    "User-Agent": "pyhole/%s" % version.version()
})


def get(url, **kwargs):
    """GET a URL."""
    return session.get(url, **kwargs)


def post(url, **kwargs):
    """POST to a URL."""
    return session.post(url, **kwargs)


def put(url, **kwargs):
    """PUT to a URL."""
    return session.put(url, **kwargs)


def ok(request):
    """Check if a request is OK (2xx)."""
    return request.status_code >= 200 and request.status_code < 300
